﻿using System.Collections.Generic;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Marketing;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class BannerRepository : IBannerRepository
    {
        #region Private

        private const string RepositoryName = "BannerRepository";

        private const string GetOneByIDSignature = "GetOneByID";
        private const string GetAllByStoreIDSignature = "GetAllByStoreID";
        private const string SearchBannerSignature = "SearchBanner";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetTableSchemaSignature = "GetTableSchema";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "Banner", "BannerLocale" } );
        private BannerRepositoryNoCache _repository = new BannerRepositoryNoCache();

        #endregion

        #region Constructors

        public BannerRepository()
        {
        }

        public BannerRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public Banner Save( Banner banner )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Banner result = _repository.Save( banner );
            _dataCache.ClearRepositoryCache();

            return result;
        }

        public void Delete( string bannerID )
        {
            _repository.Delete( bannerID );
            _dataCache.ClearRepositoryCache();
        }

        public Banner GetOneByID( Culture culture, string bannerID )
        {
            Banner result = ( Banner ) _dataCache.GetValue( GetOneByIDSignature, _cacheDependency, culture.CultureID, bannerID );

            if ( result == null )
            {
                result = _repository.GetOneByID( culture, bannerID );
                _dataCache.SetValue( GetOneByIDSignature, result, culture.CultureID, bannerID );
            }

            return result;
        }

        public IList<Banner> GetAllByStoreID( Culture culture, string storeID, BoolFilter isEnabled, string sortBy )
        {
            IList<Banner> list = ( IList<Banner> ) _dataCache.GetValue(
                GetAllByStoreIDSignature, _cacheDependency, culture.CultureID, storeID, isEnabled.ToString(), sortBy );

            if ( list == null )
            {
                list = _repository.GetAllByStoreID( culture, storeID, isEnabled, sortBy );
                _dataCache.SetValue( GetAllByStoreIDSignature, list, culture.CultureID, storeID, isEnabled.ToString(), sortBy );
            }

            return list;
        }

        public IList<Banner> SearchBanner(
            Culture culture,
            string sortBy,
            string storeID,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            howManyItems = 0;
            Pair result = ( Pair ) _dataCache.GetValue(
                SearchBannerSignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                storeID,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if ( result == null )
            {
                result = new Pair();
                result.First = _repository.SearchBanner(
                    culture, sortBy, storeID, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;

                _dataCache.SetValue(
                    SearchBannerSignature,
                    result,
                    culture.CultureID,
                    sortBy,
                    storeID,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex
                    );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return ( IList<Banner> ) result.First;
        }

        public void UpdateBannerSortOrder( string storeID, string[] sortedBanner )
        {
            _repository.UpdateBannerSortOrder( storeID, sortedBanner );
            _dataCache.ClearRepositoryCache();
        }

        public IList<ILocale> GetLocales( string bannerID )
        {
            IList<ILocale> list = ( IList<ILocale> ) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, bannerID );

            if ( list == null )
            {
                list = _repository.GetLocales( bannerID );
                _dataCache.SetValue( GetLocalesSignature, list, bannerID );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = ( IList<TableSchemaItem> ) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if ( list == null )
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        #endregion
    }
}
