using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using Vevo.Domain.Contents;
using Vevo.Domain.Products;

namespace Vevo.Domain
{
    public class GoogleXmlSitemaps
    {
        #region Private

        private string _storefrontUrl = String.Empty;
        private string _changefreq = String.Empty;
        private string _priority = String.Empty;

        private List<Category> _categories = new List<Category>();
        private List<Product> _products = new List<Product>();
        private List<Content> _contentPages = new List<Content>();

        private void GenerateElement( XmlWriter writer, string url )
        {
            writer.WriteStartElement( "url" );
            writer.WriteElementString( "loc", url );
            writer.WriteElementString( "changefreq", Changefreq );
            writer.WriteElementString( "priority", Priority );
            writer.WriteEndElement();
        }

        #endregion

        #region Public

        private void GenerateCategories( XmlWriter writer, IList<Category> categoryList )
        {
            foreach (Category category in categoryList)
            {
                if (category.IsEnabled)
                {
                    string url = StorefrontUrl + category.UrlName + "-list.aspx";
                    GenerateElement( writer, url );
                }
            }
        }

        private void GenerateProducts( XmlWriter writer, IList<Product> productList )
        {
            foreach (Product product in productList)
            {
                if (product.IsEnabled)
                {
                    string url = StorefrontUrl + product.UrlName + "-details.aspx";
                    GenerateElement( writer, url );
                }
            }
        }

        private void GenerateContentPages( XmlWriter writer, IList<Content> contentList )
        {
            foreach (Content content in contentList)
            {
                if (content.IsShowInSiteMap)
                {
                    string url = StorefrontUrl + content.UrlName + "-content.aspx";
                    GenerateElement( writer, url );
                }
            }
        }

        public static XmlWriter CreateXmlWriter( string filePhysicalPathName )
        {
            XmlWriterSettings xmlSetting = new XmlWriterSettings();
            xmlSetting.Encoding = System.Text.Encoding.UTF8;
            xmlSetting.Indent = true;
            return XmlWriter.Create( filePhysicalPathName, xmlSetting );
        }

        public void AddCategories( IList<Category> categories )
        {
            _categories.AddRange( categories );
        }

        public void AddProducts( IList<Product> products )
        {
            _products.AddRange( products );
        }

        public void AddContentPages( IList<Content> contents )
        {
            _contentPages.AddRange( contents );
        }

        public bool GenerateSitemap( XmlWriter xmlWriter, out string errorMessage )
        {
            bool success = true;
            errorMessage = String.Empty;
            try
            {
                xmlWriter.WriteStartDocument();
                xmlWriter.WriteStartElement( "urlset", "http://www.sitemaps.org/schemas/sitemap/0.9" );
                xmlWriter.WriteAttributeString( "xmlns", "xsi", null, "http://base.google.com/ns/1.0" );
                xmlWriter.WriteAttributeString( "xsi",
                    "schemaLocation",
                    null,
                    "http://www.sitemaps.org/schemas/sitemap/0.9 http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd" );
                GenerateCategories( xmlWriter, _categories );
                GenerateProducts( xmlWriter, _products );
                GenerateContentPages( xmlWriter, _contentPages );

                xmlWriter.WriteEndElement();
                xmlWriter.WriteEndDocument();
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );

                errorMessage = ex.Message;
                success = false;

            }
            finally
            {
                xmlWriter.Close();
            }
            
            return success;

        }


        public string Changefreq
        {
            set { _changefreq = value; }
            get { return _changefreq; }
        }

        public string Priority
        {
            set { _priority = value; }
            get { return _priority; }
        }

        public string StorefrontUrl
        {
            set { _storefrontUrl = value; }
            get { return _storefrontUrl; }
        }

        #endregion

    }
}
