using System;
using System.Data;
using System.Configuration;
using System.IO;
using Vevo.Domain;
using Vevo.Domain.Discounts;
using Vevo.Domain.Marketing;
using Vevo.Domain.Orders;
using Vevo.Domain.Payments;
using Vevo.Domain.Products;
using Vevo.Domain.Tax;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;
using Vevo.Domain.Stores;
using Vevo.Base.Domain;

namespace Vevo.Domain.Orders
{
    /// <summary>
    /// Summary description for OrderCreateService
    /// </summary>
    public class OrderCreateService
    {
        #region Public Enums

        public enum RecurringUnit
        {
            day,
            week,
            month,
            year
        }

        public enum RecurringDate
        {
            RegularBegin,
            RegularEnd,
            TrialBegin,
            TrialEnd
        }

        #endregion


        #region Private

        private const int MaximumRecurringDescriptionLength = 254;

        private Cart _shoppingCart;
        private CheckoutDetails _checkoutDetails;
        private Culture _culture;
        private Currency _currency;
        private string _affiliateCode;
        private string _visitorIP;
        private RecurringPaymentResult _recurringPaymentResult;


        private string ItemRecurringPeriod( ICartItem cartItem )
        {
            if (cartItem.RecurringNumberOfTrialCycles > 0)
                return "Trial";
            else
                return "Regular";
        }



        private string GetOrderItemName(
            ICartItem cartItem,
            Customer customer )
        {
            OrderCalculator orderCalculator = new OrderCalculator();
            OrderAmountPackage orderAmountPackage = orderCalculator.CalculatePackage(
                _checkoutDetails,
                new CartItemGroup( cartItem ),
                customer );

            return cartItem.GetOrderItemName(
                _culture, _currency, orderAmountPackage );
        }


        private string CreateRecurringDescription( ICartItem cartItem, OrderAmountPackage orderAmountPackage )
        {
            string description = String.Format(
                "RegularShipping: {0:f2}, RegularTax: {1:f2}, Discount: {2:f2}",
                orderAmountPackage.RecurringRegularAmount.ShippingCost + orderAmountPackage.RecurringRegularAmount.HandlingFee,
                orderAmountPackage.RecurringRegularAmount.Tax,
                orderAmountPackage.RecurringRegularAmount.Discount );

            if (cartItem.RecurringNumberOfTrialCycles > 0)
            {
                description += String.Format(
                    ", TrialShipping: {0:f2}, TrialDiscount: {1:f2}, TrialTax: {2:f2}",
                    orderAmountPackage.RecurringTrialAmount.ShippingCost + orderAmountPackage.RecurringTrialAmount.HandlingFee,
                    orderAmountPackage.RecurringTrialAmount.Discount,
                    orderAmountPackage.RecurringTrialAmount.Tax );
            }

            if (description.Length > MaximumRecurringDescriptionLength)
                description = String.Format( "{0}...", description.Substring( 0, MaximumRecurringDescriptionLength - 3 ) );

            return description;
        }

        private void ProcessPayPalIpn( string ipnID, Order order )
        {
            PayPalIpn payPalIpn = DataAccessContext.PayPalIpnRepository.GetOne( ipnID );
            payPalIpn.IsRecurringProfileCreated = true;
            DataAccessContext.PayPalIpnRepository.Save( payPalIpn );

            PaymentLog paymentLog = new PaymentLog();
            paymentLog.OrderID = order.OrderID;
            paymentLog.PaymentResponse = payPalIpn.IpnResponse;
            paymentLog.PaymentGateway = "PayPal Pro";
            paymentLog.PaymentType = "PayPalRecurringCreate";
            DataAccessContext.PaymentLogRepository.Save( paymentLog );
        }

        private RecurringProfile CreateRecurringProfile(
            ICartItem cartItem,
            OrderAmountPackage orderAmountPackage,
            RecurringPaymentResult paymentRecurringResult,
            int index,
            Order order )
        {
            RecurringProfile recurringProfile = new RecurringProfile();
            recurringProfile.ReferenceID = paymentRecurringResult.GetRecurringIDByCartItemID( cartItem.CartItemID );

            recurringProfile.RecurringStatus = SystemConst.RecurringStatus.New.ToString();
            recurringProfile.RecurringInterval = cartItem.RecurringInterval;
            recurringProfile.RecurringIntervalUnit = cartItem.RecurringIntervalUnit;
            recurringProfile.RecurringNumberOfCycles = cartItem.RecurringNumberOfCycles;
            recurringProfile.RecurringNumberOfTrialCycles = cartItem.RecurringNumberOfTrialCycles;
            recurringProfile.SetRecurringPeriod(
                cartItem.RecurringInterval,
                cartItem.RecurringIntervalUnit,
                cartItem.RecurringNumberOfCycles,
                cartItem.RecurringNumberOfTrialCycles );

            recurringProfile.RecurringSequence = 0;
            recurringProfile.UpdateTime = DateTime.Now;

            recurringProfile.TrialPrice = orderAmountPackage.RecurringTrialAmount.Total;
            recurringProfile.RegularPrice = orderAmountPackage.RecurringRegularAmount.Total;
            recurringProfile.RecurringDescription = CreateRecurringDescription( cartItem, orderAmountPackage );
            recurringProfile.RegularShipping =
                orderAmountPackage.RecurringRegularAmount.ShippingCost
                + orderAmountPackage.RecurringRegularAmount.HandlingFee;
            recurringProfile.RegularTax = orderAmountPackage.RecurringRegularAmount.Tax;
            recurringProfile.RegularDiscount = orderAmountPackage.RecurringRegularAmount.Discount;
            recurringProfile.TrialShipping =
                orderAmountPackage.RecurringTrialAmount.ShippingCost
                + orderAmountPackage.RecurringTrialAmount.HandlingFee;
            recurringProfile.TrialTax = orderAmountPackage.RecurringTrialAmount.Tax;
            recurringProfile.TrialDiscount = orderAmountPackage.RecurringTrialAmount.Discount;

            recurringProfile = DataAccessContext.RecurringProfileRepository.Save( recurringProfile );

            string ipnID = DataAccessContext.PayPalIpnRepository.GetIpnIDByReferenceID( recurringProfile.ReferenceID );
            if (!String.IsNullOrEmpty( ipnID ))
            {
                ProcessPayPalIpn( ipnID, order );

                recurringProfile.RecurringStatus = SystemConst.RecurringStatus.CreateVerified.ToString();
                recurringProfile = DataAccessContext.RecurringProfileRepository.Save( recurringProfile );
            }

            return recurringProfile;
        }

        private OrderItem CreateOrderItem(
            string orderID,
            ICartItem cartItem,
            Customer customer,
            RecurringProfile recurringProfile,
            string storeID
            )
        {
            OrderItem orderItem = new OrderItem();
            Product product = DataAccessContext.ProductRepository.GetOne( _culture, cartItem.ProductID, storeID );
            orderItem.OrderID = orderID;
            orderItem.ProductID = cartItem.ProductID;
            orderItem.Name = GetOrderItemName( cartItem, customer );
            orderItem.Quantity = cartItem.Quantity;
            orderItem.Sku = product.Sku;
            orderItem.Model = product.Model;
            orderItem.UnitPrice = cartItem.GetCheckoutUnitPrice( customer.GetWholesaleStatus() );
            orderItem.GiftRegistryItemID =
                ConvertUtilities.ToInt32(
                    _checkoutDetails.CartItemIDToGiftRegistryIDMap[cartItem.CartItemID] ).ToString();
            orderItem.UploadFile = cartItem.Options.GetUploadedListString( _culture, _currency );

            if (recurringProfile != null)
            {
                orderItem.RecurringID = recurringProfile.RecurringID;
                orderItem.OrderItemRecurring.RecurringStatus = SystemConst.RecurringStatus.New.ToString();
                orderItem.OrderItemRecurring.RecurringPeriod = ItemRecurringPeriod( cartItem );
                orderItem.OrderItemRecurring.UpdateTime = DateTime.Now;
            }

            return orderItem;
        }

        private void MoveUploadedFilesToActualFolder(
            OrderItem orderItem,
            ICartItem cartItem )
        {
            foreach (OptionItemValue optionItemValue in cartItem.Options)
            {
                if (optionItemValue.TempUploadedFile != null)
                {
                    string actualFileName = orderItem.GenerateUploadedFileName(
                        optionItemValue.GetDisplayedName( _culture, _currency ) );
                    string[] filenames = actualFileName.Split( new string[] { " (" }, StringSplitOptions.None );
                    string outputFilename = String.Empty;
                    if (filenames[filenames.Length - 1].EndsWith( ")" ))
                    {
                        for (int i = 0; i < filenames.Length - 1; i++)
                            outputFilename += filenames[i];
                    }
                    else
                    {
                        outputFilename = actualFileName;
                    }
                    optionItemValue.TempUploadedFile.MoveToActualFolder( outputFilename );
                }
            }
        }

        private void CreateGiftCertificates( OrderItem orderItem, ICartItem cartItem )
        {
            if (cartItem.IsGiftCertificate)
            {
                GiftCertificateProduct product = (GiftCertificateProduct) cartItem.Product;
                for (int i = 0; i < cartItem.Quantity; i++)
                {
                    GiftCertificate giftCert =
                        product.GenerateGiftCertificate( orderItem.OrderItemID, cartItem.GiftDetails );

                    DataAccessContext.GiftCertificateRepository.Create( giftCert );
                }
            }
        }

        private void UpdateProductStock( ICartItem cartItem, Culture culture, string storeID )
        {
            if (!cartItem.IsPromotion)
            {
                Product product = cartItem.Product;
                product.DeductStock( cartItem.Quantity, cartItem.Options.GetUseStockOptionItemIDs() );
                if (product.IsProductKit)
                {
                    foreach (ProductKitItemValue item in cartItem.ProductKits)
                    {
                        Product productItem = DataAccessContext.ProductRepository.GetOne( culture, item.ProductID, storeID );
                        productItem.DeductStock( item.Quantity * cartItem.Quantity, new string[0] );
                        DataAccessContext.ProductRepository.Save( productItem );
                    }
                }
                DataAccessContext.ProductRepository.Save( product );
            }
            //else
            //{
            //    foreach (PromotionSelectedItem productItem in cartItem.PromotionSelected.PromotionSelectedItems)
            //    {
            //        Product product = productItem.Product;
            //        product.DeductStock( productItem.GetPromotionProduct.Quantity * cartItem.Quantity, productItem.GetUseStockOptionItems().ToArray( typeof( string ) ) as string[] );
            //        DataAccessContext.ProductRepository.Save( product );
            //    }
            //}

        }

        private void UpdateCouponUsage( string couponID )
        {
            if (!String.IsNullOrEmpty( couponID ))
            {
                Coupon coupon = DataAccessContext.CouponRepository.GetOne( couponID );
                coupon.CurrentQuantity += 1;
                DataAccessContext.CouponRepository.Update( coupon );
            }
        }

        private void UpdateGiftCertificateRemainValue(
            OrderAmount orderAmount )
        {
            if (!_checkoutDetails.GiftCertificate.IsNull)
            {
                GiftCertificate giftCertificate = _checkoutDetails.GiftCertificate;

                giftCertificate.RemainValue -= orderAmount.GiftCertificate;
                if (giftCertificate.RemainValue < 0)
                    giftCertificate.RemainValue = 0;

                DataAccessContext.GiftCertificateRepository.Update( giftCertificate );
            }
        }


        private bool HasAnyRecurringProduct()
        {
            foreach (ICartItem cartItem in _shoppingCart.CartItems)
            {
                if (cartItem.IsRecurring)
                    return true;
            }
            return false;
        }

        private Order CreateOrder(
            Order order,
            Customer customer,
            OrderAmount orderAmount,
            string storeID,
            Culture culture
            )
        {
            order.ContainsRecurring = HasAnyRecurringProduct();
            order = DataAccessContext.OrderRepository.Save( order );

            int index = 0;
            foreach (ICartItem cartItem in _shoppingCart.CartItems)
            {
                RecurringProfile recurringProfile = RecurringProfile.Null;
                if (cartItem.IsRecurring)
                {
                    CartItemGroup cartItemGroup = new CartItemGroup( cartItem );
                    OrderCalculator orderCalculator = new OrderCalculator();
                    OrderAmountPackage orderAmountPackage = orderCalculator.CalculatePackage(
                        _checkoutDetails, cartItemGroup, customer );

                    recurringProfile = CreateRecurringProfile(
                        cartItem, orderAmountPackage, _recurringPaymentResult, index, order );

                    index++;
                }

                OrderItem orderItem = CreateOrderItem( order.OrderID, cartItem, customer, recurringProfile, storeID );
                orderItem = DataAccessContext.OrderItemRepository.Save( orderItem );

                MoveUploadedFilesToActualFolder( orderItem, cartItem );

                CreateGiftCertificates( orderItem, cartItem );

                UpdateProductStock( cartItem, culture, storeID );
            }

            UpdateCouponUsage( order.CouponID );
            UpdateGiftCertificateRemainValue( orderAmount );

            return order;
        }

        #endregion


        #region Constructors

        public OrderCreateService(
            Cart shoppingCart,
            CheckoutDetails checkoutDetails,
            Culture culture,
            Currency currency,
            string affiliateCode,
            string visitorIP,
            RecurringPaymentResult recurringPaymentResult )
        {
            _shoppingCart = shoppingCart;
            _checkoutDetails = checkoutDetails;
            _culture = culture;
            _currency = currency;
            _affiliateCode = affiliateCode;
            _visitorIP = visitorIP;
            _recurringPaymentResult = recurringPaymentResult;
        }

        public OrderCreateService(
            Cart shoppingCart,
            CheckoutDetails checkoutDetails,
            Culture culture,
            Currency currency,
            string affiliateCode,
            string visitorIP )
        {
            _shoppingCart = shoppingCart;
            _checkoutDetails = checkoutDetails;
            _culture = culture;
            _currency = currency;
            _affiliateCode = affiliateCode;
            _visitorIP = visitorIP;
        }

        #endregion


        #region Public Methods

        public OrderAmount GetOrderAmount(
            Customer customer )
        {
            OrderCalculator orderCalculator = new OrderCalculator();
            return orderCalculator.Calculate(
                _checkoutDetails,
                _shoppingCart.SeparateCartItemGroups(),
                customer );
        }

        public OrderAmountPackage GetOrderAmountPackage(
            Customer customer )
        {
            OrderCalculator orderCalculator = new OrderCalculator();
            return orderCalculator.CalculatePackage(
                _checkoutDetails,
                _shoppingCart.GetAllCartItems(),
                customer );
        }

        public Order PlaceOrderAnonymous(
            OrderAmount orderAmount,
            string userName,
            Address billingAddress,
            string email,
            IStoreRetriever storeRetriever,
            Culture culture
            )
        {
            //OrderAmount orderAmount = GetOrderAmount( Customer.Null );

            Order order = _checkoutDetails.CreateOrderAnonymous(
                userName, billingAddress, email, _currency, orderAmount, _visitorIP, storeRetriever );

            return CreateOrder( order, Customer.Null, orderAmount, storeRetriever.GetCurrentStoreID(), culture );
        }

        public Order PlaceOrder(OrderAmount orderAmount,
            Customer customer, IStoreRetriever storeRetriever, Culture culture
            )
        {
            //OrderAmount orderAmount = GetOrderAmount( customer );

            Order order = _checkoutDetails.CreateOrder(
                customer, _currency, orderAmount, _visitorIP, storeRetriever );

            return CreateOrder( order, customer, orderAmount, storeRetriever.GetCurrentStoreID(), culture );
        }

        #endregion

    }
}
