using System;
using System.Collections.Generic;
using Vevo.Domain.Base;
using Vevo.Domain.Exceptions;
using Vevo.Domain.Stores;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;
using Vevo.Domain.Configurations;
using Vevo.Base.Domain;

namespace Vevo.Domain.Products
{
    // Product class is an Aggregate Root consisting of: 
    //      Product, ProductLocale, ProductOptionGroup, ProductImage, ProductCategory,
    //      ProductRecurring, ProductShippingCost,
    //      ProductStock, OptionCombination, 
    [Serializable]
    public class Product : ILocaleLoader
    {
        #region Private

        private string _productID = "0";
        private string _sku = String.Empty;
        private string _model = String.Empty;
        private string _imageSecondary = String.Empty;
        private double _weight;
        private bool _fixedShippingCost = false;
        private bool _freeShippingCost = false;
        private decimal _shippingCost;
        private string _manufacturer = String.Empty;
        private string _keywords = String.Empty;
        private IList<String> _categoryIDs = new List<String>();
        private IList<String> _departmentIDs = new List<String>();
        private bool _isTodaySpecial;
        private int _minQuantity = 1;
        private int _maxQuantity = 0;
        private bool _isDowmloadable;
        private string _downloadPath = String.Empty;
        private bool _isEnabled = true;
        private string _relatedProducts = String.Empty;
        private double _merchantRating;
        private string _discountGroupID = "0";
        public bool _isGiftCertificate;
        public bool _isFixedPrice = true;
        public string _manufacturerPartNumber = String.Empty;
        public string _upc = String.Empty;
        private string _taxClassID = "0";
        private bool _isParentVisible = true;
        private string _other1 = String.Empty;
        private string _other2 = String.Empty;
        private string _other3 = String.Empty;
        private string _other4 = String.Empty;
        private string _other5 = String.Empty;
        private bool _useInventory = true;
        private string _productDetailsLayoutPath = String.Empty;
        private string _QBListID = String.Empty;
        private bool _isQBExported = false;

        private bool _isRecurring;
        private bool _isAffiliate;
        private bool _isCustomPrice;
        private bool _isCallForPrice = false;
        private string _brand = String.Empty;
        private int _returnTime = 0;

        private bool _isProductKit = false;
        private bool _isDynamicProductKitPrice = false;
        private bool _isDynamicProductKitWeight = false;

        private DateTime _createDate = DateTime.Now;
        private int _viewCount = 0;

        private Culture _defaultCulture;
        private LocaleCollection<ProductLocale> _locales;
        private ProductRecurring _productRecurring;
        private ProductCustomPrice _productCustomPrice;
        private MetaInformationCollection<ProductMetaInformation> _metaInformation;

        //------------------------------------------------------------
        // Custom private members

        private IEntityList<ProductImage> _productImages;
        private IList<ProductOptionGroup> _productOptionGroups;
        private IList<ProductStock> _productStocks;
        private IEntityList<ProductShippingCost> _productShippingCosts;
        private IList<ProductPrice> _productPrices;
        private IList<ProductSpecification> _productSpecifications;
        private IList<ProductKit> _productKits;

        private string GetMatchingOptionItemID( OptionGroup optionGroup, IList<String> inventoryOptionItemIDs )
        {
            foreach (OptionItem optionItem in optionGroup.OptionItems)
            {
                if (inventoryOptionItemIDs.Contains( optionItem.OptionItemID ))
                    return optionItem.OptionItemID;
            }

            return null;
        }

        // For each inventory option item, it needs to associate with one option group.

        private void AssertStockOptionItemIDs( string[] optionItemIDs )
        {
            IList<String> optionItemIDList = new List<String>( optionItemIDs );

            foreach (ProductOptionGroup productOptionGroup in ProductOptionGroups)
            {
                if (productOptionGroup.UseStock)
                {
                    string optionItemID = GetMatchingOptionItemID(
                        productOptionGroup.OptionGroup, optionItemIDList );

                    if (optionItemID == null)
                        throw new DomainException(
                            "Cannot match Option Group '" + productOptionGroup.OptionGroup.Name
                            + "' with inventory option item." );

                    optionItemIDList.Remove( optionItemID );
                }
            }

            if (optionItemIDList.Count > 0)
                throw new DomainException(
                    "Invalid option item ID " + optionItemIDList[0]
                    + ". It cannot be used for inventory tracking." );
        }

        private bool IsOutOfStock( object currentStock, object useInventory )
        {

            if (DataAccessContext.Configurations.GetBoolValue( "UseStockControl" ) && ConvertUtilities.ToBoolean( useInventory ))
            {
                if (ConvertUtilities.ToInt32( currentStock ) <= DataAccessContext.Configurations.GetIntValue( "OutOfStockValue" ))
                    return true;
                else
                    return false;
            }
            else
                return false;


        }

        private IList<string> GetUseStockOptionGroupIDs()
        {
            List<string> list = new List<string>();
            foreach (ProductOptionGroup productOptionGroup in ProductOptionGroups)
            {
                if (productOptionGroup.UseStock)
                {
                    list.Add( productOptionGroup.OptionGroupID );
                }
            }
            return list;
        }

        private void SetStockRecursively(
            string currentOptionItemIDs,
            IList<string> useStockOptionGroupIDs,
            int depth )
        {
            IList<OptionItem> optionItems = DataAccessContext.OptionItemRepository.GetByOptionGroupID(
                DefaultCulture, useStockOptionGroupIDs[depth] );

            foreach (OptionItem optionItem in optionItems)
            {
                string newOptionItemIDs = currentOptionItemIDs + "," + optionItem.OptionItemID;

                if (depth == useStockOptionGroupIDs.Count - 1)
                {
                    // This is the final option group, set stock value of accumulated data
                    string[] idList = newOptionItemIDs.Split(
                        new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries );
                    SetStock( 0, idList );
                }
                else
                {
                    SetStockRecursively( newOptionItemIDs, useStockOptionGroupIDs, depth + 1 );
                }
            }

        }

        private void SetStockForAllOptionItems( IList<string> useStockOptionGroupIDs )
        {
            SetStockRecursively( String.Empty, useStockOptionGroupIDs, 0 );
        }

        // Refresh ProductStock objects if using "Use Stock" option
        // Suppose that there are three option groups, called GroupA, GroupB, GroupC.
        //   Then, GroupA has two option items: A1, A2
        //   GroupB has two option items: B1, B2
        //   GroupC has two option items: C1, C2
        //
        // Then, Product1 uses all three groups and set "Use Stock" for GroupA and GroupC.
        // The ProductStock objects for this Product1 should be combinations of all
        // option items in GroupA and GroupC, which are:
        //      A1-C1
        //      A1-C2
        //      A2-C1
        //      A2-C2
        // Totally four ProductStock objects for Product1. The initial stock numbers
        // for all four ProductStock objects are zero (0).
        private void RefreshProductStocks()
        {
            ProductStocks.Clear();
            IList<string> useStockOptionGroupIDs = GetUseStockOptionGroupIDs();
            SetStockForAllOptionItems( useStockOptionGroupIDs );
        }

        private ProductMetaInformation GetMetaInformationByCultureStoreID( Culture culture, string storeID, bool isProtect )
        {
            foreach (ProductMetaInformation meta in _metaInformation)
            {
                if ((meta.CultureID == culture.CultureID) && (meta.StoreID == storeID))
                {
                    return meta;
                }
            }
            if ((storeID != "0") || (isProtect == false))
            {
                return null;
            }
            else
            {
                return AddDefaultValueMetaInformation( culture );
            }
        }

        private ProductMetaInformation AddDefaultValueMetaInformation( Culture culture )
        {
            ProductMetaInformation meta = new ProductMetaInformation();
            meta.CultureID = culture.CultureID;
            meta.ProductID = this.ProductID;
            meta.StoreID = "0";
            _metaInformation.Add( meta );
            return meta;
        }

        private IList<ProductSpecification> GetSpecificationItem( string specificationID )
        {
            List<ProductSpecification> list = new List<ProductSpecification>();
            foreach (ProductSpecification item in ProductSpecifications)
            {
                if (item.SpecificationItemID == specificationID)
                    list.Add( item );
            }
            return list;
        }

        #endregion


        #region Public Constructors

        public Product( Culture defaultCulture )
        {
            _defaultCulture = defaultCulture;
            _locales = new LocaleCollection<ProductLocale>( this );
            _metaInformation = new MetaInformationCollection<ProductMetaInformation>( this );
            _productCustomPrice = new ProductCustomPrice();
        }

        #endregion


        #region Public Properties

        public static Product Null
        {
            get { return NullProduct.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string ProductID { get { return _productID; } set { _productID = value; } }
        public string Sku { get { return _sku; } set { _sku = value; } }
        public string Model { get { return _model; } set { _model = value; } }
        public string ImageSecondary { get { return _imageSecondary; } set { _imageSecondary = value; } }
        public double Weight { get { return _weight; } set { _weight = value; } }
        public bool FixedShippingCost { get { return _fixedShippingCost; } set { _fixedShippingCost = value; } }
        public bool FreeShippingCost { get { return _freeShippingCost; } set { _freeShippingCost = value; } }
        public decimal ShippingCost { get { return _shippingCost; } set { _shippingCost = value; } }
        public string Manufacturer { get { return _manufacturer; } set { _manufacturer = value; } }
        public string Keywords { get { return _keywords; } set { _keywords = value; } }
        public IList<String> CategoryIDs { get { return _categoryIDs; } set { _categoryIDs = value; } }
        public IList<String> DepartmentIDs { get { return _departmentIDs; } set { _departmentIDs = value; } }

        public bool IsTodaySpecial { get { return _isTodaySpecial; } set { _isTodaySpecial = value; } }
        public int MinQuantity { get { return _minQuantity; } set { _minQuantity = value; } }
        public int MaxQuantity { get { return _maxQuantity; } set { _maxQuantity = value; } }
        public bool IsDownloadable { get { return _isDowmloadable; } set { _isDowmloadable = value; } }
        public string DownloadPath { get { return _downloadPath; } set { _downloadPath = value; } }
        public bool IsEnabled { get { return _isEnabled; } set { _isEnabled = value; } }
        public string RelatedProducts { get { return _relatedProducts; } set { _relatedProducts = value; } }
        public double MerchantRating { get { return _merchantRating; } set { _merchantRating = value; } }
        public string DiscountGroupID { get { return _discountGroupID; } set { _discountGroupID = value; } }
        public bool IsGiftCertificate { get { return _isGiftCertificate; } set { _isGiftCertificate = value; } }
        public bool IsFixedPrice { get { return _isFixedPrice; } set { _isFixedPrice = value; } }
        public string ManufacturerPartNumber { get { return _manufacturerPartNumber; } set { _manufacturerPartNumber = value; } }
        public string Upc { get { return _upc; } set { _upc = value; } }
        public string TaxClassID { get { return _taxClassID; } set { _taxClassID = value; } }
        public bool IsParentVisible { get { return _isParentVisible; } set { _isParentVisible = value; } }
        public string Other1 { get { return _other1; } set { _other1 = value; } }
        public string Other2 { get { return _other2; } set { _other2 = value; } }
        public string Other3 { get { return _other3; } set { _other3 = value; } }
        public string Other4 { get { return _other4; } set { _other4 = value; } }
        public string Other5 { get { return _other5; } set { _other5 = value; } }
        public bool UseInventory { get { return _useInventory; } set { _useInventory = value; } }
        public string ProductDetailsLayoutPath { get { return _productDetailsLayoutPath; } set { _productDetailsLayoutPath = value; } }

        public bool IsRecurring { get { return _isRecurring; } set { _isRecurring = value; } }
        public bool IsAffiliate { get { return _isAffiliate; } set { _isAffiliate = value; } }
        public bool IsCustomPrice { get { return _isCustomPrice; } set { _isCustomPrice = value; } }
        public bool IsCallForPrice { get { return _isCallForPrice; } set { _isCallForPrice = value; } }
        public string Brand { get { return _brand; } set { _brand = value; } }
        public int ReturnTime { get { return _returnTime; } set { _returnTime = value; } }

        public bool IsProductKit { get { return _isProductKit; } set { _isProductKit = value; } }
        public bool IsDynamicProductKitPrice { get { return _isDynamicProductKitPrice; } set { _isDynamicProductKitPrice = value; } }
        public bool IsDynamicProductKitWeight { get { return _isDynamicProductKitWeight; } set { _isDynamicProductKitWeight = value; } }

        public Culture DefaultCulture { get { return _defaultCulture; } set { _defaultCulture = value; } }
        public LocaleCollection<ProductLocale> Locales { get { return _locales; } }
        public string QBListID { get { return _QBListID; } set { _QBListID = value; } }
        public bool IsQBExported { get { return _isQBExported; } set { _isQBExported = value; } }
        public DateTime CreateDate { get { return _createDate; } set { _createDate = value; } }
        public int ViewCount { get { return _viewCount; } set { _viewCount = value; } }
        public ProductRecurring ProductRecurring
        {
            get { return _productRecurring; }
            set
            {
                _productRecurring = value;
            }
        }
        public ProductCustomPrice ProductCustomPrice
        {
            get { return _productCustomPrice; }
            set { _productCustomPrice = value; }
        }

        public MetaInformationCollection<ProductMetaInformation> MetaInformations { get { return _metaInformation; } }

        //------------------------------------------------------------
        // Locale items
        public string Name
        {
            get { return Locales[DefaultCulture].Name; }
            set { Locales.GetNoLoad( DefaultCulture ).Name = value; }
        }
        public string ShortDescription
        {
            get { return Locales[DefaultCulture].ShortDescription; }
            set { Locales.GetNoLoad( DefaultCulture ).ShortDescription = value; }
        }
        public string LongDescription
        {
            get { return Locales[DefaultCulture].LongDescription; }
            set { Locales.GetNoLoad( DefaultCulture ).LongDescription = value; }
        }
        public string UrlName
        {
            get { return Locales[DefaultCulture].UrlName; }
            set { Locales.GetNoLoad( DefaultCulture ).UrlName = value; }
        }
        // End Locale items
        //------------------------------------------------------------

        #endregion


        #region Custom Properties

        public virtual bool HasTrialPeriod
        {
            get
            {
                if (IsRecurring)
                    return ProductRecurring.HasTrialPeriod;
                else
                    return false;
            }
        }

        //------------------------------------------------------------
        // Data from related tables (e.g. ProductStock, OptionGroups)

        public int SumStock
        {
            get
            {
                int sumStock = 0;
                foreach (ProductStock productStock in ProductStocks)
                    sumStock += productStock.Stock;

                return sumStock;
            }
        }

        public IEntityList<ProductImage> ProductImages
        {
            get
            {
                if (_productImages == null)
                    _productImages = new EntityList<ProductImage>(
                        DataAccessContext.ProductRepository.GetProductImages( ProductID ) );
                return _productImages;
            }
        }

        public IList<ProductOptionGroup> ProductOptionGroups
        {
            get
            {
                if (_productOptionGroups == null)
                    _productOptionGroups =
                        DataAccessContext.ProductRepository.GetProductOptionGroups( DefaultCulture, ProductID );

                return _productOptionGroups;
            }
        }

        public IList<ProductStock> ProductStocks
        {
            get
            {
                if (_productStocks == null)
                    _productStocks =
                        DataAccessContext.ProductRepository.GetProductStocks( ProductID );

                return _productStocks;
            }
        }

        public IEntityList<ProductShippingCost> ProductShippingCosts
        {
            get
            {
                if (_productShippingCosts == null)
                    _productShippingCosts = new EntityList<ProductShippingCost>(
                        DataAccessContext.ProductRepository.GetProductShippingCosts( ProductID ) );

                return _productShippingCosts;
            }
        }

        public IList<ProductPrice> ProductPrices
        {
            get
            {
                if (_productPrices == null)
                    _productPrices =
                        DataAccessContext.ProductRepository.GetProductPrices( ProductID );

                return _productPrices;
            }
            set
            {
                _productPrices = value;
            }
        }

        public IList<ProductSpecification> ProductSpecifications
        {

            get
            {
                if (_productSpecifications == null)
                    _productSpecifications =
                        DataAccessContext.ProductRepository.GetProductSpecifications( ProductID );

                return _productSpecifications;
            }
            set
            {
                _productSpecifications = value;
            }
        }

        public IList<ProductKit> ProductKits
        {
            get
            {
                if (_productKits == null)
                    _productKits =
                        DataAccessContext.ProductRepository.GetProductKits( DefaultCulture, ProductID );

                return _productKits;
            }
            set
            {
                _productKits = value;
            }
        }

        #endregion


        #region Public Methods

        public string GetPageTitle( Culture culture, Store store )
        {
            string title = SeoVariable.ReplaceProductVariable(
                DataAccessContext.ProductRepository.GetOne( culture, ProductID, store.StoreID ),
                culture,
                DataAccessContext.Configurations.GetValue( culture.CultureID, "DefaultProductPageTitle", store ) );

            if (String.IsNullOrEmpty( title ))
                return _locales[culture].Name;
            else
                return title;
        }

        public ProductMetaInformation GetMetaInformation( Culture culture, string storeID )
        {
            return GetMetaInformationByCultureStoreID( culture, storeID, false );
        }

        public ProductMetaInformation GetMetaInformation( string storeID )
        {
            return GetMetaInformationByCultureStoreID( DefaultCulture, storeID, false );
        }

        public string GetMetaKeyword( Culture culture, string storeID )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( culture, storeID, true );
            if (meta != null)
            {
                if (meta.UseDefaultMetaKeyword)
                    meta.MetaKeyword = GetMetaInformationByCultureStoreID( culture, "0", true ).MetaKeyword;

                if (!String.IsNullOrEmpty( meta.MetaKeyword ))
                {
                    return meta.MetaKeyword;
                }
                else
                {
                    return _locales[culture].Name;

                }
            }
            else

                return _locales[culture].Name;
        }

        public string GetMetaKeyword( string storeID )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( DefaultCulture, storeID, true );
            if (meta != null)
            {
                if (meta.UseDefaultMetaKeyword)
                    meta.MetaKeyword = GetMetaInformationByCultureStoreID( DefaultCulture, "0", true ).MetaKeyword;

                if (!String.IsNullOrEmpty( meta.MetaKeyword ))
                {
                    return meta.MetaKeyword;
                }
                else
                {
                    return _locales[DefaultCulture].Name;

                }
            }
            else

                return _locales[DefaultCulture].Name;
        }

        public bool GetUseDefaultValueMetaKeyword( Culture culture, string storeID )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( culture, storeID, true );
            if (meta != null)
            {
                if (meta.UseDefaultMetaKeyword)
                    meta.MetaKeyword = GetMetaInformationByCultureStoreID( culture, "0", true ).MetaKeyword;
                return meta.UseDefaultMetaKeyword;
            }
            else
                return true;
        }

        public bool GetUseDefaultValueMetaKeyword( string storeID )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( DefaultCulture, storeID, true );
            if (meta != null)
            {
                if (meta.UseDefaultMetaKeyword)
                    meta.MetaKeyword = GetMetaInformationByCultureStoreID( DefaultCulture, "0", true ).MetaKeyword;
                return meta.UseDefaultMetaKeyword;
            }
            else
                return true;
        }

        public void SetMetaInfo( string storeID, string metaKeyword, string metaDescription, bool isNew )
        {
            if (isNew)
            {
                ProductMetaInformation meta = new ProductMetaInformation();
                meta.CultureID = DefaultCulture.CultureID;
                meta.StoreID = storeID;
                meta.MetaKeyword = metaKeyword;
                meta.MetaDescription = metaDescription;
                _metaInformation.Add( meta );
            }
            else
            {
                SetMetaInfo( storeID, metaKeyword, metaDescription );
            }
        }
        public void SetMetaInfo( string storeID, string metaKeyword, string metaDescription )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( DefaultCulture, storeID, true );
            if (meta == null)
            {
                meta = new ProductMetaInformation();
                meta.CultureID = DefaultCulture.CultureID;
                meta.StoreID = storeID;
                meta.MetaKeyword = metaKeyword;
                meta.MetaDescription = metaDescription;
                _metaInformation.Add( meta );
            }
            else
            {
                meta.MetaKeyword = metaKeyword;
                meta.MetaDescription = metaDescription;
            }

        }

        public void SetMetaInfo(
            string storeID, string metaKeyword, string metaDescription, bool metaKeyUseDefaultValue, bool metaDesUseDefaultValue )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( DefaultCulture, storeID, true );

            if (meta == null)
            {
                meta = new ProductMetaInformation();
                meta.CultureID = DefaultCulture.CultureID;
                meta.StoreID = storeID;
                meta.MetaKeyword = metaKeyword;
                meta.MetaDescription = metaDescription;
                meta.UseDefaultMetaKeyword = metaKeyUseDefaultValue;
                meta.UseDefaultMetaDescription = metaDesUseDefaultValue;

                _metaInformation.Add( meta );
            }
            else
            {
                meta.MetaKeyword = metaKeyword;
                meta.MetaDescription = metaDescription;
                meta.UseDefaultMetaKeyword = metaKeyUseDefaultValue;
                meta.UseDefaultMetaDescription = metaDesUseDefaultValue;
            }

        }

        public void SetMetaKeyword( Culture culture, string storeID, string metaKeyword )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( culture, storeID, true );
            if (meta == null)
            {
                meta = new ProductMetaInformation();
                meta.CultureID = culture.CultureID;
                meta.StoreID = storeID;
                meta.MetaKeyword = metaKeyword;
                _metaInformation.Add( meta );
            }
            else
            {
                meta.MetaKeyword = metaKeyword;
            }
        }

        public void SetUseDefaultValueMetaKeyword( Culture culture, string storeID, bool useDefaultValue )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( culture, storeID, true );
            if (meta == null)
            {
                meta = new ProductMetaInformation();
                meta.CultureID = culture.CultureID;
                meta.StoreID = storeID;
                meta.UseDefaultMetaKeyword = useDefaultValue;
                _metaInformation.Add( meta );
            }
            else
            {
                meta.UseDefaultMetaKeyword = useDefaultValue;
            }
        }

        public void SetMetaKeyword( string storeID, string metaKeyword )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( DefaultCulture, storeID, true );
            if (meta == null)
            {
                meta = new ProductMetaInformation();
                meta.CultureID = DefaultCulture.CultureID;
                meta.StoreID = storeID;
                meta.MetaKeyword = metaKeyword;
                _metaInformation.Add( meta );
            }
            else
            {
                meta.MetaKeyword = metaKeyword;
            }
        }

        public void SetUseDefaultValueMetaKeyword( string storeID, bool useDefaultValue )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( DefaultCulture, storeID, true );
            if (meta == null)
            {
                meta = new ProductMetaInformation();
                meta.CultureID = DefaultCulture.CultureID;
                meta.StoreID = storeID;
                meta.UseDefaultMetaKeyword = useDefaultValue;
                _metaInformation.Add( meta );
            }
            else
            {
                meta.UseDefaultMetaKeyword = useDefaultValue;
            }
        }

        public void SetProductSpecificationItemValue( string specificationItemID, string value )
        {
            IList<ProductSpecification> items = GetSpecificationItem( specificationItemID );
            if (items.Count == 0)
            {
                ProductSpecification item = new ProductSpecification();
                item.SpecificationItemID = specificationItemID;
                item.Value = value;
                ProductSpecifications.Add( item );
            }
            else
            {
                foreach (ProductSpecification spec in ProductSpecifications)
                {
                    if (spec.SpecificationItemID == specificationItemID)
                        spec.Value = value;
                }
            }
        }

        public string GetMetaDescription( Culture culture, string storeID )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( culture, storeID, true );
            if (meta != null)
            {
                if (meta.UseDefaultMetaDescription)
                    meta.MetaDescription = GetMetaInformationByCultureStoreID( culture, "0", true ).MetaDescription;

                if (!String.IsNullOrEmpty( meta.MetaDescription ))
                {
                    return meta.MetaDescription;
                }
                else
                {
                    return _locales[culture].ShortDescription;

                }
            }
            else
                return _locales[culture].ShortDescription;

        }

        public string GetMetaDescription( string storeID )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( DefaultCulture, storeID, true );
            if (meta != null)
            {
                if (meta.UseDefaultMetaDescription)
                    meta.MetaDescription =
                        GetMetaInformationByCultureStoreID( DefaultCulture, "0", true ).MetaDescription;

                if (!String.IsNullOrEmpty( meta.MetaDescription ))
                {
                    return meta.MetaDescription;
                }
                else
                {
                    return _locales[DefaultCulture].ShortDescription;

                }
            }
            else

                return _locales[DefaultCulture].ShortDescription;
        }

        public bool GetUseDefaultValueMetaDescription( Culture culture, string storeID )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( culture, storeID, true );
            if (meta != null)
            {
                if (meta.UseDefaultMetaDescription)
                    meta.MetaDescription = GetMetaInformationByCultureStoreID( culture, "0", true ).MetaDescription;
                return meta.UseDefaultMetaDescription;
            }
            else
                return true;
        }

        public bool GetUseDefaultValueMetaDescription( string storeID )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( DefaultCulture, storeID, true );
            if (meta != null)
            {
                if (meta.UseDefaultMetaDescription)
                    meta.MetaDescription = GetMetaInformationByCultureStoreID( DefaultCulture, "0", true ).MetaDescription;
                return meta.UseDefaultMetaDescription;
            }
            else
                return true;
        }

        public void SetMetaDescription( Culture culture, string storeID, string metaDescription )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( culture, storeID, true );
            if (meta == null)
            {
                meta = new ProductMetaInformation();
                meta.CultureID = culture.CultureID;
                meta.StoreID = storeID;
                meta.MetaDescription = metaDescription;
                _metaInformation.Add( meta );
            }
            else
            {
                meta.MetaDescription = metaDescription;
            }
        }

        public void SetUseDefaultValueMetaDescription( Culture culture, string storeID, bool useDefaultValue )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( culture, storeID, true );
            if (meta == null)
            {
                meta = new ProductMetaInformation();
                meta.CultureID = culture.CultureID;
                meta.StoreID = storeID;
                meta.UseDefaultMetaDescription = useDefaultValue;
                _metaInformation.Add( meta );
            }
            else
            {
                meta.UseDefaultMetaDescription = useDefaultValue;
            }
        }

        public void SetMetaDescription( string storeID, string metaDescription )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( DefaultCulture, storeID, true );
            if (meta == null)
            {
                meta = new ProductMetaInformation();
                meta.CultureID = DefaultCulture.CultureID;
                meta.StoreID = storeID;
                meta.MetaDescription = metaDescription;
                _metaInformation.Add( meta );
            }
            else
            {
                meta.MetaDescription = metaDescription;
            }
        }

        public void SetUseDefaultValueMetaDescription( string storeID, bool useDefaultValue )
        {
            ProductMetaInformation meta = GetMetaInformationByCultureStoreID( DefaultCulture, storeID, true );
            if (meta == null)
            {
                meta = new ProductMetaInformation();
                meta.CultureID = DefaultCulture.CultureID;
                meta.StoreID = storeID;
                meta.UseDefaultMetaDescription = useDefaultValue;
                _metaInformation.Add( meta );
            }
            else
            {
                meta.UseDefaultMetaDescription = useDefaultValue;
            }
        }

        public IList<ILocale> GetLocales()
        {
            List<ILocale> result = new List<ILocale>();

            foreach (ProductLocale locale in DataAccessContext.ProductRepository.GetLocales( ProductID ))
                result.Add( locale );

            return result;
        }

        public IList<ILocale> GetMetaInformations( string storeID )
        {
            List<ILocale> result = new List<ILocale>();

            foreach (ProductMetaInformation metaInformation in DataAccessContext.ProductRepository.GetMetaInformationsByStoreID( ProductID, storeID ))
                result.Add( metaInformation );

            return result;
        }

        public IList<ILocale> GetMetaInformations()
        {
            List<ILocale> result = new List<ILocale>();

            foreach (ProductMetaInformation metaInformation in DataAccessContext.ProductRepository.GetMetaInformations( ProductID ))
                result.Add( metaInformation );

            return result;
        }

        // Copy the whole aggregate (including locale)
        public virtual object Clone()
        {
            Product newObject = (Product) MemberwiseClone();
            newObject._locales = (LocaleCollection<ProductLocale>) Locales.Clone();
            newObject._productImages = ProductImages.Clone();
            newObject._metaInformation = (MetaInformationCollection<ProductMetaInformation>) MetaInformations.Clone();

            return newObject;
        }

        public void AddOptionGroup( OptionGroup optionGroup, bool useStock )
        {
            ProductOptionGroup productOptionGroup = new ProductOptionGroup( DefaultCulture );
            productOptionGroup.OptionGroupID = optionGroup.OptionGroupID.ToString();
            productOptionGroup.UseStock = useStock;

            ProductOptionGroups.Add( productOptionGroup );

            if (useStock)
            {
                RefreshProductStocks();
            }
        }

        public void AddProductKitGroup( ProductKitGroup productKitGroup, int sortOrder )
        {
            ProductKit productKit = new ProductKit( DefaultCulture );
            productKit.ProductKitGroupID = productKitGroup.ProductKitGroupID;
            productKit.SortOrder = sortOrder;
            ProductKits.Add( productKit );
        }

        public ProductStock FindProductStock( params string[] optionItemIDs )
        {
            foreach (ProductStock productStock in ProductStocks)
            {
                if (ListUtilities<string>.AreSameSet( productStock.OptionItemIDs, optionItemIDs ))
                    return productStock;
            }

            return null;
        }

        public int GetStock( params string[] optionItemIDs )
        {
            if (optionItemIDs.Length == 1 && optionItemIDs[0].ToString() == "0")
                optionItemIDs = new string[] { };

            AssertStockOptionItemIDs( optionItemIDs );

            ProductStock productStock = FindProductStock( optionItemIDs );
            if (productStock == null)
                return 0;
            else
                return productStock.Stock;
        }

        public void SetStock( int stock, params string[] optionItemIDs )
        {
            AssertStockOptionItemIDs( optionItemIDs );

            ProductStock productStock = FindProductStock( optionItemIDs );
            if (productStock == null)
            {
                productStock = new ProductStock( optionItemIDs );
                productStock.Stock = stock;
                ProductStocks.Add( productStock );
            }
            else
            {
                productStock.Stock = stock;
            }
        }

        public void SetStockWithNoOption( int stock )
        {
            ProductStock productStock = FindProductStock();
            if (productStock == null)
            {
                productStock = new ProductStock();
                productStock.Stock = stock;
                ProductStocks.Add( productStock );
            }
            else
            {
                productStock.Stock = stock;
            }
        }

        public void DeductStock( int deductQuantity, params string[] optionItemIDs )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseStockControl" )
                && UseInventory)
            {
                int current = GetStock( optionItemIDs );
                if (current < deductQuantity)
                    throw new DomainException(
                        "Stock quantity is too small. Stock: " + current + "  Deduct: " + deductQuantity );

                SetStock( current - deductQuantity, optionItemIDs );
            }
        }

        public decimal GetOverriddenShippingCost( string shippingID )
        {
            foreach (ProductShippingCost productShippingCost in ProductShippingCosts)
            {
                if (productShippingCost.ShippingID == shippingID)
                    return productShippingCost.FixedShippingCost;
            }

            return 0;
        }

        public virtual decimal GetDisplayedPrice( WholesaleStatus wholesaleStatus )
        {
            if (DataAccessContext.StoreRetriever.IsStorefrontUrl())
                return GetDisplayedPrice( wholesaleStatus, DataAccessContext.StoreRetriever.GetCurrentStoreID() );
            else
                throw new Exception( "The function supports multi-store and requires store ID parameter." );

        }

        public virtual decimal GetDisplayedPrice( WholesaleStatus wholesaleStatus, string storeID )
        {
            ProductPrice productPrice = GetProductPrice( storeID );

            if (IsFixedPrice)
                return wholesaleStatus.GetPrice(
                    productPrice.Price, productPrice.WholesalePrice, productPrice.WholesalePrice2, productPrice.WholesalePrice3 );
            else
                return wholesaleStatus.GetPricePercentageOnly( productPrice.Price );
        }


        public virtual decimal GetCheckoutPrice( WholesaleStatus wholesaleStatus )
        {
            return GetCheckoutPrice( wholesaleStatus, DataAccessContext.StoreRetriever.GetCurrentStoreID() );
        }

        public virtual decimal GetCheckoutPrice( WholesaleStatus wholesaleStatus, string storeID )
        {
            ProductPrice productPrice = GetProductPrice( storeID );
            return GetCheckoutPrice( wholesaleStatus, productPrice.Price, storeID );
        }

        public virtual decimal GetCheckoutPrice( WholesaleStatus wholesaleStatus, decimal userEnteredAmount )
        {
            return GetCheckoutPrice( wholesaleStatus, userEnteredAmount, DataAccessContext.StoreRetriever.GetCurrentStoreID() );
        }

        public virtual decimal GetCheckoutPrice( WholesaleStatus wholesaleStatus, decimal userEnteredAmount, string storeID )
        {
            ProductPrice productPrice = GetProductPrice( storeID );

            if (IsCustomPrice)
            {
                return userEnteredAmount;
            }

            if (IsFixedPrice)
            {
                if (IsRecurring
                    && ProductRecurring.HasTrialPeriod)
                {
                    return wholesaleStatus.GetPricePercentageOnly( ProductRecurring.RecurringTrialAmount );
                }
                else
                {
                    return wholesaleStatus.GetPrice(
                        productPrice.Price, productPrice.WholesalePrice, productPrice.WholesalePrice2, productPrice.WholesalePrice3 );
                }
            }
            else
            {
                return wholesaleStatus.GetPricePercentageOnly( userEnteredAmount );
            }
        }


        public ProductImage GetPrimaryProductImage()
        {
            foreach (ProductImage productImage in ProductImages)
            {
                if (productImage.SortOrder == 0)
                    return productImage;
            }
            return new ProductImage();
        }

        public ProductImage GetProductImage( string productImageID )
        {
            foreach (ProductImage productImage in ProductImages)
            {
                if (productImage.ProductImageID == productImageID)
                    return productImage;
            }
            return new ProductImage();
        }

        public decimal GetRecurringRegularPrice( WholesaleStatus wholesaleStatus )
        {
            return GetRecurringRegularPrice( wholesaleStatus, DataAccessContext.StoreRetriever.GetCurrentStoreID() );
        }

        public decimal GetRecurringRegularPrice( WholesaleStatus wholesaleStatus, string storeID )
        {
            ProductPrice productPrice = GetProductPrice( storeID );
            return wholesaleStatus.GetPrice( productPrice.Price, productPrice.WholesalePrice, productPrice.WholesalePrice2, productPrice.WholesalePrice3 );
        }

        public decimal GetRecurringTrialPrice( WholesaleStatus wholesaleStatus )
        {
            if (HasTrialPeriod)
                return wholesaleStatus.GetPricePercentageOnly( ProductRecurring.RecurringTrialAmount );
            else
                return 0;
        }

        public bool IsOutOfStock()
        {
            if (UseInventory)
            {
                foreach (ProductStock stock in ProductStocks)
                {
                    if (stock.Stock <= DataAccessContext.Configurations.GetIntValue( "OutOfStockValue" ))
                        return true;
                }
                return false;
            }
            else
                return false;
        }

        public int GetPurchasableStock( int numberItemInSession, params string[] optionItemIDs )
        {
            return GetStock( optionItemIDs ) - numberItemInSession;
        }

        public bool IsOutOfStock( int purchaseQuantity, int numberItemInSession, params string[] optionItemIDs )
        {
            int remainStock = GetPurchasableStock( numberItemInSession, optionItemIDs ) - purchaseQuantity;

            if (remainStock == DataAccessContext.Configurations.GetIntValue( "OutOfStockValue" ))
                return false;
            else if (IsOutOfStock( remainStock, UseInventory ))
                return true;
            else
                return false;
        }

        // If true, this product cannot be added to cart directly.
        // User needs to enter some information in the product details page first.
        public virtual bool RequiresUserInput()
        {
            if (ProductOptionGroups.Count > 0)
                return true;
            else
                return false;
        }

        public ProductPrice GetProductPrice( string storeID )
        {
            bool isNewProductPrice;

            return GetProductPrice( storeID, out isNewProductPrice );
        }

        public ProductPrice GetProductPrice( string storeID, out bool isNewProductPrice )
        {
            isNewProductPrice = false;

            foreach (ProductPrice productPrice in ProductPrices)
            {
                if (productPrice.StoreID == storeID)
                    return productPrice;
            }

            isNewProductPrice = true;
            return new ProductPrice();
        }

        public void SetPrice(
            string storeID,
            decimal price,
            decimal retailprice,
            decimal wholesalePrice,
            decimal wholesalePrice2,
            decimal wholesalePrice3 )
        {
            ProductPrice productPrice = GetProductPrice( storeID );

            SetPrice( storeID, price, retailprice, wholesalePrice, wholesalePrice2, wholesalePrice3,
                productPrice.UseDefaultPrice,
                productPrice.UseDefaultRetailPrice,
                productPrice.UseDefaultWholeSalePrice,
                productPrice.UseDefaultWholeSalePrice2,
                productPrice.UseDefaultWholeSalePrice3 );
        }

        public void SetPrice(
            string storeID,
            decimal price,
            decimal retailprice,
            decimal wholesalePrice,
            decimal wholesalePrice2,
            decimal wholesalePrice3,
            bool isUseDefaultPrice,
            bool isUseDefaultRetailPrice,
            bool isUseDefaultWholesalePrice,
            bool isUseDefaultWholesalePrice2,
            bool isUseDefaultWholesalePrice3 )
        {
            bool isNewProductPrice;

            ProductPrice productPrice = GetProductPrice( storeID, out isNewProductPrice );
            productPrice.Price = price;
            productPrice.RetailPrice = retailprice;
            productPrice.WholesalePrice = wholesalePrice;
            productPrice.WholesalePrice2 = wholesalePrice2;
            productPrice.WholesalePrice3 = wholesalePrice3;
            productPrice.UseDefaultPrice = isUseDefaultPrice;
            productPrice.UseDefaultRetailPrice = isUseDefaultRetailPrice;
            productPrice.UseDefaultWholeSalePrice = isUseDefaultWholesalePrice;
            productPrice.UseDefaultWholeSalePrice2 = isUseDefaultWholesalePrice2;
            productPrice.UseDefaultWholeSalePrice3 = isUseDefaultWholesalePrice3;

            if (isNewProductPrice)
            {
                productPrice.StoreID = storeID;
                ProductPrices.Add( productPrice );
            }
        }

        public void SetProductPrice( string storeID, decimal price )
        {
            bool isNewProductPrice;

            ProductPrice productPrice = GetProductPrice( storeID, out isNewProductPrice );
            productPrice.Price = price;

            if (isNewProductPrice)
            {
                productPrice.StoreID = storeID;
                ProductPrices.Add( productPrice );
            }
        }

        public void SetPrice( string storeID, ProductPrice inputPrice, bool isNew )
        {
            if (isNew)
            {
                ProductPrice productPrice = new ProductPrice();
                productPrice.Price = inputPrice.Price;
                productPrice.RetailPrice = inputPrice.RetailPrice;
                productPrice.WholesalePrice = inputPrice.WholesalePrice;
                productPrice.WholesalePrice2 = inputPrice.WholesalePrice2;
                productPrice.WholesalePrice3 = inputPrice.WholesalePrice3;
                productPrice.UseDefaultPrice = inputPrice.UseDefaultPrice;
                productPrice.UseDefaultRetailPrice = inputPrice.UseDefaultRetailPrice;
                productPrice.UseDefaultWholeSalePrice = inputPrice.UseDefaultWholeSalePrice;
                productPrice.UseDefaultWholeSalePrice2 = inputPrice.UseDefaultWholeSalePrice2;
                productPrice.UseDefaultWholeSalePrice3 = inputPrice.UseDefaultWholeSalePrice3;

                if (!KeyUtilities.IsMultistoreLicense())
                {
                    productPrice.UseDefaultPrice = true;
                    productPrice.UseDefaultRetailPrice = true;
                    productPrice.UseDefaultWholeSalePrice = true;
                    productPrice.UseDefaultWholeSalePrice2 = true;
                    productPrice.UseDefaultWholeSalePrice3 = true;
                }

                productPrice.StoreID = storeID;
                ProductPrices.Add( productPrice );

            }
            else
            {
                SetPrice( storeID, inputPrice );
            }
        }
        public void SetPrice( string storeID, ProductPrice inputPrice )
        {
            bool isNewProductPrice;

            ProductPrice productPrice = GetProductPrice( storeID, out isNewProductPrice );
            productPrice.Price = inputPrice.Price;
            productPrice.RetailPrice = inputPrice.RetailPrice;
            productPrice.WholesalePrice = inputPrice.WholesalePrice;
            productPrice.WholesalePrice2 = inputPrice.WholesalePrice2;
            productPrice.WholesalePrice3 = inputPrice.WholesalePrice3;
            productPrice.UseDefaultPrice = inputPrice.UseDefaultPrice;
            productPrice.UseDefaultRetailPrice = inputPrice.UseDefaultRetailPrice;
            productPrice.UseDefaultWholeSalePrice = inputPrice.UseDefaultWholeSalePrice;
            productPrice.UseDefaultWholeSalePrice2 = inputPrice.UseDefaultWholeSalePrice2;
            productPrice.UseDefaultWholeSalePrice3 = inputPrice.UseDefaultWholeSalePrice3;

            if (!KeyUtilities.IsMultistoreLicense())
            {
                productPrice.UseDefaultPrice = true;
                productPrice.UseDefaultRetailPrice = true;
                productPrice.UseDefaultWholeSalePrice = true;
                productPrice.UseDefaultWholeSalePrice2 = true;
                productPrice.UseDefaultWholeSalePrice3 = true;
            }

            if (isNewProductPrice)
            {
                productPrice.StoreID = storeID;
                ProductPrices.Add( productPrice );
            }
        }

        public bool IsAvailable( string rootCategoryID )
        {
            foreach (String categoryID in CategoryIDs)
            {
                Category category = DataAccessContext.CategoryRepository.GetOne( DefaultCulture, categoryID );

                if (category.RootID == rootCategoryID)
                    return true;
            }

            return false;
        }

        public bool IsProductAvailable( string storeID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( DefaultCulture, _productID, storeID );
            IList<String> storeList = product.GetAvailableStoreIDList();
            IList<String> categoryList = product.CategoryIDs;
            IList<String> departmentList = product.DepartmentIDs;
            bool isCategoryAvailable = false;
            bool isDepartmentAvailable = false;

            foreach (string categoryID in categoryList)
            {
                Category category = DataAccessContext.CategoryRepository.GetOne( DefaultCulture, categoryID );

                if (category.IsEnabled)
                {
                    isCategoryAvailable = true;
                    break;
                }
            }

            foreach (string departmentID in departmentList)
            {
                Department department = DataAccessContext.DepartmentRepository.GetOne( DefaultCulture, departmentID );

                if (department.IsEnabled)
                {
                    isDepartmentAvailable = true;
                    break;
                }
            }

            if (product.IsNull || !product.IsEnabled || !storeList.Contains( storeID ) || (!isCategoryAvailable && !isDepartmentAvailable))
                return false;

            return true;
        }

        public IList<string> GetAvailableStoreIDList()
        {
            IList<string> rootCategoryIDs = new List<string>();
            for (int i = 0; i < CategoryIDs.Count; i++)
            {
                Category category = DataAccessContext.CategoryRepository.GetOne( DefaultCulture, CategoryIDs[i] );
                rootCategoryIDs.Add( category.RootID );
            }

            IList<string> rootDepartmentIDs = new List<string>();
            for (int i = 0; i < DepartmentIDs.Count; i++)
            {
                Department department = DataAccessContext.DepartmentRepository.GetOne( DefaultCulture, DepartmentIDs[i] );
                rootDepartmentIDs.Add( department.RootID );
            }

            IList<Store> storeList = DataAccessContext.StoreRepository.GetAll( "StoreID" );
            IList<string> storeIDList = new List<string>();

            for (int i = 0; i < storeList.Count; i++)
            {
                if (storeList[i].StoreID != "0")
                {
                    foreach (string rootCategoryID in rootCategoryIDs)
                    {
                        if (rootCategoryID ==
                            DataAccessContext.Configurations.GetValue( "RootCategory", storeList[i] ))
                        {
                            storeIDList.Add( storeList[i].StoreID );
                        }
                    }

                    foreach (string rootDepartmentID in rootDepartmentIDs)
                    {
                        if (rootDepartmentID == DataAccessContext.Configurations.GetValue( "RootDepartment", storeList[i] ))
                        {
                            storeIDList.Add( storeList[i].StoreID );
                        }
                    }
                }
            }

            return storeIDList;
        }

        public string GetSpecificationItemValue( string specificationID )
        {
            IList<ProductSpecification> items = GetSpecificationItem( specificationID );
            if (items.Count > 0)
            {
                string values = string.Empty;
                foreach (ProductSpecification spec in items)
                    values += spec.Value + ",";
                return values.Substring( 0, values.Length - 1 );
            }
            else
                return "-";
        }

        public string GetSpecificationGroupID( Culture culture )
        {
            SpecificationItem item = DataAccessContext.SpecificationItemRepository.GetOne(
                culture, ProductSpecifications[0].SpecificationItemID );

            return item.SpecificationGroupID;
        }

        #endregion
    }
}
